#include "arrangement.h"

using namespace std;

namespace Msp {
namespace GLtk {

Arrangement::Arrangement(Layout &l):
	layout(l),
	parent(layout.get_arrangement())
{
	layout.push_arrangement(*this);
}

Arrangement::~Arrangement()
{
	layout.pop_arrangement(*this);
}

void Arrangement::arrange(Widget &wdg)
{
	for(unsigned i=0; i<4; ++i)
		process_widget(wdg, static_cast<Side>(i), true);
	finish_widget(wdg);
	finish_slot();
}

void Arrangement::arrange(Arrangement &arr)
{
	for(unsigned i=0; i<4; ++i)
	{
		Side side = static_cast<Side>(i);
		const Edge &edge = arr.get_edge(side);
		for(list<Widget *>::const_iterator j=edge.widgets.begin(); j!=edge.widgets.end(); ++j)
			process_widget(**j, side, edge.aligned);
	}
	finish_slot();
}

void Arrangement::add_constraint(Widget &wdg, Layout::ConstraintType type, Side side)
{
	add_constraint(wdg, type, edges[side]);
}

void Arrangement::add_constraint(Widget &wdg, Layout::ConstraintType type, const Edge &edge)
{
	for(list<Widget *>::const_iterator i=edge.widgets.begin(); i!=edge.widgets.end(); ++i)
		if(*i!=&wdg)
			layout.add_constraint(wdg, type, **i);
}

Layout::ConstraintType Arrangement::get_order_constraint(Side s, bool slack)
{
	switch(s)
	{
	case TOP:    return (slack ? Layout::FAR_ABOVE : Layout::ABOVE);
	case RIGHT:  return (slack ? Layout::FAR_RIGHT_OF : Layout::RIGHT_OF);
	case BOTTOM: return (slack ? Layout::FAR_BELOW : Layout::BELOW);
	case LEFT:   return (slack ? Layout::FAR_LEFT_OF : Layout::LEFT_OF);
	default: throw invalid_argument("Arrangement::get_align_constraint");
	}
}

Layout::ConstraintType Arrangement::get_align_constraint(Side s)
{
	switch(s)
	{
	case TOP:    return Layout::ALIGN_TOP;
	case RIGHT:  return Layout::ALIGN_RIGHT;
	case BOTTOM: return Layout::ALIGN_BOTTOM;
	case LEFT:   return Layout::ALIGN_LEFT;
	default: throw invalid_argument("Arrangement::get_align_constraint");
	}
}


Arrangement::Edge::Edge():
	aligned(false)
{ }

void Arrangement::Edge::clear()
{
	widgets.clear();
	aligned = false;
}

void Arrangement::Edge::add(Widget &wdg, bool algn)
{
	if(aligned)
		return;

	if(algn)
		widgets.clear();

	widgets.push_back(&wdg);
	aligned = algn;
}

void Arrangement::Edge::align()
{
	if(aligned || widgets.empty())
		return;

	list<Widget *>::iterator second = widgets.begin();
	++second;

	widgets.erase(second, widgets.end());
	aligned = true;
}

} // namespace GLtk
} // namespace Msp
