#include <msp/gl/matrix.h>
#include <msp/gl/meshbuilder.h>
#include "graphic.h"
#include "part.h"
#include "style.h"
#include "vslider.h"

namespace Msp {
namespace GLtk {

VSlider::VSlider():
	slider_size(1)
{ }

void VSlider::autosize()
{
	if(!style)
		return;

	Widget::autosize();

	if(const Part *slider_part = style->get_part("slider"))
	{
		const Sides &margin = slider_part->get_margin();
		const Geometry &pgeom = slider_part->get_geometry();
		geom.w = std::max(geom.w, pgeom.w+margin.left+margin.right);
		geom.h = std::max(geom.h, pgeom.h*3/2+margin.top+margin.bottom);
	}
}

void VSlider::rebuild_special(const Part &part, CachedPart &cache)
{
	if(part.get_name()=="slider")
	{
		const Graphic *graphic = part.get_graphic(state);
		if(!graphic)
			return;

		Alignment align = part.get_alignment();
		if(max>min)
			align.y = (value-min)/(max-min);

		Geometry pgeom = part.get_geometry();
		align.apply(pgeom, geom, part.get_margin());

		cache.texture = part.get_graphic(state)->get_texture();
		cache.clear_mesh();

		GL::MeshBuilder bld(*cache.mesh);
		bld.matrix() *= GL::Matrix::translation(pgeom.x, pgeom.y, 0);
		graphic->build(pgeom.w, pgeom.h, bld);
	}
}

void VSlider::button_press(int x, int y, unsigned btn)
{
	if(btn==1 && geom.is_inside_relative(x, y) && max>min)
	{
		int sy = static_cast<int>((geom.h-slider_size)*(value-min)/(max-min));
		if(y<sy)
			set_value(value-step*10);
		else if(y>=static_cast<int>(sy+slider_size))
			set_value(value+step*10);
		else
			start_drag(y);
	}
}

void VSlider::button_release(int, int, unsigned btn)
{
	if(btn==1 && dragging)
		end_drag();
}

void VSlider::pointer_motion(int, int y)
{
	if(dragging)
		drag(y);
}

void VSlider::on_geometry_change()
{
	drag_area_size = geom.h-slider_size;
}

void VSlider::on_style_change()
{
	if(!style)
		return;

	if(const Part *slider_part = style->get_part("slider"))
		slider_size = slider_part->get_geometry().h;

	on_geometry_change();
}

} // namespace GLtk
} // namespace Msp

